/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.client;

import java.util.Map;

import org.keycloak.representations.idm.ClientRepresentation;
import org.keycloak.representations.idm.ProtocolMapperRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

@Mapper(unmappedTargetPolicy = ReportingPolicy.IGNORE, unmappedSourcePolicy = ReportingPolicy.IGNORE, collectionMappingStrategy = CollectionMappingStrategy.ACCESSOR_ONLY)
interface KeycloakClientMapper {

	public static final String PROTOCOL = "openid-connect";
	public static final String ATTRIBUTE_NAME_POST_LOGOUT_REDIRECT_URIS = "post.logout.redirect.uris";
	public static final String ATTRIBUTE_VALUE_POST_LOGOUT_REDIRECT_URIS = "+";

	@Mapping(target = "protocol", constant = PROTOCOL)
	@Mapping(target = "publicClient", constant = "true")
	@Mapping(target = "directAccessGrantsEnabled", constant = "true")
	@Mapping(target = "standardFlowEnabled", constant = "true")
	@Mapping(target = "implicitFlowEnabled", constant = "true")
	@Mapping(target = "bearerOnly", constant = "false")
	@Mapping(target = "attributes", source = ".", qualifiedByName = "mapAttributes")
	ClientRepresentation map(OzgKeycloakClientSpec client);

	@Mapping(target = "protocol", constant = PROTOCOL)
	@Mapping(target = "publicClient", constant = "true")
	@Mapping(target = "directAccessGrantsEnabled", constant = "true")
	@Mapping(target = "standardFlowEnabled", constant = "true")
	@Mapping(target = "implicitFlowEnabled", constant = "true")
	@Mapping(target = "bearerOnly", constant = "false")
	@Mapping(target = "attributes", source = ".", qualifiedByName = "mapAttributes")
	ClientRepresentation update(@MappingTarget ClientRepresentation existingClient, OzgKeycloakClientSpec spec);

	@Named("mapAttributes")
	default Map<String, String> mapAttributes(OzgKeycloakClientSpec client) {
		return Map.of(ATTRIBUTE_NAME_POST_LOGOUT_REDIRECT_URIS, ATTRIBUTE_VALUE_POST_LOGOUT_REDIRECT_URIS);
	}

	RoleRepresentation mapRole(OzgKeycloakClientSpec.ClientRole spec);

	RoleRepresentation updateRole(@MappingTarget RoleRepresentation existingRole, OzgKeycloakClientSpec.ClientRole spec);

	@Mapping(target = "protocolMapper", source = "type")
	ProtocolMapperRepresentation mapProtocolMapper(OzgKeycloakClientSpec.ProtocolMapper mapper);
}
