package de.ozgcloud.operator.keycloak.user;

import java.time.Duration;
import java.util.Optional;

import de.ozgcloud.operator.keycloak.OzgCustomResourceStatus;
import io.javaoperatorsdk.operator.api.reconciler.UpdateControl;

class UserUpdateControlBuilder {

	private OzgKeycloakUser resource;

	private OzgCustomResourceStatus status;
	private Optional<String> message = Optional.empty();

	private boolean reschedule = false;
	private Duration scheduleDuration;

	public UserUpdateControlBuilder(OzgKeycloakUser resource) {
		this.resource = resource;
	}

	public static UserUpdateControlBuilder fromResource(OzgKeycloakUser resource) {
		return new UserUpdateControlBuilder(resource);
	}

	public UserUpdateControlBuilder withStatus(OzgCustomResourceStatus status) {
		this.status = status;
		return this;
	}

	public UserUpdateControlBuilder withMessage(String message) {
		this.message = Optional.ofNullable(message);
		return this;
	}

	public UserUpdateControlBuilder withReschedule(Duration duration) {
		this.reschedule = true;
		this.scheduleDuration = duration;
		return this;
	}

	public UpdateControl<OzgKeycloakUser> build() {
		resource.setStatus(buildOzgKeycloakUserStatus());

		return buildUpdateControl();
	}

	private OzgKeycloakUserStatus buildOzgKeycloakUserStatus() {
		var userStatus = OzgKeycloakUserStatus.builder().status(status);
		message.ifPresent(userStatus::message);

		return userStatus.build();
	}

	private UpdateControl<OzgKeycloakUser> buildUpdateControl() {
		if (reschedule) {
			return UpdateControl.updateStatus(resource).rescheduleAfter(scheduleDuration);
		}
		return UpdateControl.updateStatus(resource);
	}
}
