/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.realm;

import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.resource.RealmResource;
import org.keycloak.admin.client.resource.RealmsResource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

class KeycloakRealmRemoteServiceTest {

	@Spy
	@InjectMocks
	private KeycloakRealmRemoteService remoteService;

	@Mock
	private Keycloak keycloak;

	@Mock
	private RealmsResource realmsResource;

	@Mock
	private RealmResource realmResource;

	@Nested
	class TestCreateRealm {

		@BeforeEach
		void init() {
			when(keycloak.realms()).thenReturn(realmsResource);
		}

		@Test
		void shouldCallCreateRealm() {
			var realm = RealmRepresentationTestFactory.create();

			remoteService.createRealm(realm);

			verify(realmsResource).create(realm);
		}
	}

	@Nested
	class TestDeleteRealm {

		private static final String REALM_NAME = "TestRealmName";

		@Test
		void shouldDeleteRealm() {
			when(keycloak.realm(REALM_NAME)).thenReturn(realmResource);


			remoteService.deleteRealm(REALM_NAME);

			verify(realmResource).remove();
		}
	}
}
